"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.opSpecToKernelSpecs = exports.CodeWriter = void 0;
var expr_1 = require("./expr");
var CodeWriter = /** @class */ (function () {
    function CodeWriter() {
        this.indentLevel = 0;
        this.lines = [];
    }
    CodeWriter.prototype.indent = function () {
        this.indentLevel++;
    };
    CodeWriter.prototype.dedent = function () {
        this.indentLevel--;
    };
    CodeWriter.prototype.writeLine = function (line) {
        this.lines.push("    ".repeat(this.indentLevel) + line);
    };
    CodeWriter.prototype.toString = function () {
        return this.lines.join("\n");
    };
    return CodeWriter;
}());
exports.CodeWriter = CodeWriter;
function opSpecToKernelSpecs(op) {
    if (op.type == "reduction") {
        return getReductionKernelSpecs(op);
    }
    else if (op.type == "binary") {
        return getBinaryKernelSpecs(op);
    }
    else {
        return getUnaryKernelSpecs(op);
    }
}
exports.opSpecToKernelSpecs = opSpecToKernelSpecs;
function getReductionKernelSpecs(op) {
    var specs = [getReductionKernelSpec(op)];
    if (op.backward) {
        // specs.push(getReductionGradKernelSpec(op, op.backward));
    }
    return specs;
}
function getBinaryKernelSpecs(op) {
    var specs = [getBinaryKernelSpec(op), getBinaryInplaceKernelSpec(op)];
    if (op.backward) {
        specs.push(getBinaryGradKernelSpec(op, op.backward));
    }
    return specs;
}
function getUnaryKernelSpecs(op) {
    var specs = [getUnaryKernelSpec(op), getUnaryInplaceKernelSpec(op)];
    if (op.backward) {
        specs.push(getUnaryGradKernelSpec(op, op.backward));
    }
    return specs;
}
function getReductionKernelSpec(op) {
    var initCode = (0, expr_1.exprCodeToWebGLShader)(op.init, {
        input: "input[global_id.x]",
        output: "accumulator",
    });
    var forwardCode = (0, expr_1.exprCodeToWebGLShader)(op.forward, {
        input: "input[i]",
        output: "accumulator",
    });
    var reduceCode = op.reduce === undefined ? "" : (0, expr_1.exprCodeToWebGLShader)(op.reduce, {
        input: "input[i]",
        output: "accumulator",
    });
    var shader = "\n    var ".concat(initCode, ";\n    // Load inputData into local memory\n    for (var i = local_id.x; i < parameters.size; i += $$workgroupSize$$) {\n        ").concat(forwardCode, ";\n    }\n    // Write partial group sum to outputData\n    output[local_id.x] = accumulator;\n\n    workgroupBarrier(); // Make sure all threads have completed summation\n\n    // First thread sums up results from all other threads\n    if (local_id.x == 0u) {\n        var numToSum = min(parameters.size, $$workgroupSize$$u);\n        for (var i = 1u; i < numToSum; i++) {\n            accumulator ").concat(op.combineOp, "= output[i];\n        }\n        // Store final reduction in the first element of result array\n        ").concat(reduceCode, ";\n        output[0] = accumulator;\n    }\n");
    return {
        name: op.name,
        config: [
            {
                name: "dtype",
            },
            {
                name: "workgroupSize",
            },
        ],
        parameters: [
            {
                name: "size",
                shaderType: "u32",
            },
        ],
        inputs: [
            {
                name: "input",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "output",
                shaderType: "array<f32>",
                size: "workgroupSize",
            },
        ],
        workgroupSize: ["workgroupSize", 1, 1],
        workgroupCount: ["size/workgroupSize", 1, 1],
        shader: shader,
    };
}
function getBinaryKernelSpec(op) {
    var parameters = [
        {
            name: "size",
            shaderType: "u32",
        },
    ];
    var subs = {
        input: "input[global_id.x]",
        other: "other[global_id.x]",
        output: "output[global_id.x]",
    };
    var shaderSnippet = (0, expr_1.exprCodeToWebGLShader)(op.forward, subs);
    var shader = "\n        if (global_id.x >= parameters.size) {\n            return;\n        }\n        ".concat(shaderSnippet, ";");
    return {
        name: op.name,
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: parameters,
        inputs: [
            {
                name: "input",
                shaderType: "array<f32>",
            },
            {
                name: "other",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "output",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: [64, 1, 1],
        workgroupCount: ["size/8", 1, 1],
        shader: shader,
    };
}
function getBinaryInplaceKernelSpec(op) {
    var parameters = [
        {
            name: "size",
            shaderType: "u32",
        },
    ];
    var ast = (0, expr_1.parseCode)(op.forward);
    var subs = {
        input: "input[global_id.x]",
        other: "other[global_id.x]",
        output: "input[global_id.x]",
    };
    var shaderAst = (0, expr_1.substituteIdentifiers)(ast, subs);
    var shaderSnippet = (0, expr_1.exprNodeToWebGLShader)(shaderAst);
    var shader = "\n        if (global_id.x >= parameters.size) {\n            return;\n        }\n        ".concat(shaderSnippet, ";");
    return {
        name: op.name + "_",
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: parameters,
        inputs: [
            {
                name: "other",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "input",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: [64, 1, 1],
        workgroupCount: ["size/8", 1, 1],
        shader: shader,
    };
}
function getBinaryGradKernelSpec(op, backward) {
    var parameters = [
        {
            name: "size",
            shaderType: "u32",
        },
    ];
    var ast = (0, expr_1.parseCode)(backward);
    var subs = {
        input: "input[global_id.x]",
        inputGrad: "inputGrad[global_id.x]",
        output: "output[global_id.x]",
        outputGrad: "outputGrad[global_id.x]",
        other: "other[global_id.x]",
        otherGrad: "otherGrad[global_id.x]",
    };
    var shaderAst = (0, expr_1.substituteIdentifiers)(ast, subs);
    var shaderSnippet = (0, expr_1.exprNodeToWebGLShader)(shaderAst);
    var shader = "\n        if (global_id.x >= parameters.size) {\n            return;\n        }\n        ".concat(shaderSnippet, ";");
    return {
        name: op.name + "Grad",
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: parameters,
        inputs: [
            {
                name: "input",
                shaderType: "array<f32>",
            },
            {
                name: "other",
                shaderType: "array<f32>",
            },
            {
                name: "outputGrad",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "inputGrad",
                shaderType: "array<f32>",
                size: "size",
            },
            {
                name: "otherGrad",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: [64, 1, 1],
        workgroupCount: ["size/8", 1, 1],
        shader: shader,
    };
}
function getUnaryKernelSpec(op) {
    var parameters = [
        {
            name: "size",
            shaderType: "u32",
        },
    ];
    var ast = (0, expr_1.parseCode)(op.forward);
    var subs = {
        input: "input[global_id.x]",
        output: "output[global_id.x]",
    };
    var shaderAst = (0, expr_1.substituteIdentifiers)(ast, subs);
    var shaderSnippet = (0, expr_1.exprNodeToWebGLShader)(shaderAst);
    var shader = "\n        if (global_id.x >= parameters.size) {\n            return;\n        }\n        ".concat(shaderSnippet, ";");
    return {
        name: op.name,
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: parameters,
        inputs: [
            {
                name: "input",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "output",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: [64, 1, 1],
        workgroupCount: ["size/8", 1, 1],
        shader: shader,
    };
}
function getUnaryInplaceKernelSpec(op) {
    var parameters = [
        {
            name: "size",
            shaderType: "u32",
        },
    ];
    var ast = (0, expr_1.parseCode)(op.forward);
    var subs = {
        input: "input[global_id.x]",
        output: "input[global_id.x]",
    };
    var shaderAst = (0, expr_1.substituteIdentifiers)(ast, subs);
    var shaderSnippet = (0, expr_1.exprNodeToWebGLShader)(shaderAst);
    var shader = "\n        if (global_id.x >= parameters.size) {\n            return;\n        }\n        ".concat(shaderSnippet, ";");
    return {
        name: op.name + "_",
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: parameters,
        inputs: [],
        outputs: [
            {
                name: "input",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: [64, 1, 1],
        workgroupCount: ["size/8", 1, 1],
        shader: shader,
    };
}
function getUnaryGradKernelSpec(op, backward) {
    var parameters = [
        {
            name: "size",
            shaderType: "u32",
        },
    ];
    var ast = (0, expr_1.parseCode)(backward);
    var subs = {
        input: "input[global_id.x]",
        inputGrad: "inputGrad[global_id.x]",
        output: "output[global_id.x]",
        outputGrad: "outputGrad[global_id.x]",
    };
    var shaderAst = (0, expr_1.substituteIdentifiers)(ast, subs);
    var shaderSnippet = (0, expr_1.exprNodeToWebGLShader)(shaderAst);
    var shader = "\n        if (global_id.x >= parameters.size) {\n            return;\n        }\n        ".concat(shaderSnippet, ";");
    return {
        name: op.name + "Grad",
        config: [
            {
                name: "dtype",
            },
        ],
        parameters: parameters,
        inputs: [
            {
                name: "input",
                shaderType: "array<f32>",
            },
            {
                name: "outputGrad",
                shaderType: "array<f32>",
            },
        ],
        outputs: [
            {
                name: "inputGrad",
                shaderType: "array<f32>",
                size: "size",
            },
        ],
        workgroupSize: [64, 1, 1],
        workgroupCount: ["size/8", 1, 1],
        shader: shader,
    };
}
//# sourceMappingURL=opgen.js.map